;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Loading this file installs CG/JS.
;; Please see readme.txt for more information
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(in-package :user)

(eval-when (compile eval load)
  (require :shell)
  (use-package :excl.shell))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; validate files

(let ((platform-file
       (directory
	(merge-pathnames
	 "PLATFORM-*"
	 (path-pathname *load-pathname*)))))
  (or platform-file
      (error "Could not find the PLATFORM file."))
  
  (when (> (length platform-file) 1)
    (error "There is more than one PLATFORM file: ~a." platform-file))
  (setq platform-file (car platform-file))
  
  (or (=~ "^PLATFORM-(.*)" (file-namestring platform-file))
      (error "Cannot parse the PLATFORM file."))
  
  (let ((expected-platform $1)
	(current-platform
	 (format nil "~a~a" (symbol-name sys::.os.) #+smp "smp" #-smp "")))
    (or (string-equal expected-platform current-platform)
	(error
	 "The current platform (~a) does not match the downloaded version (~a)."
	 current-platform expected-platform)))
  ;; We are on the correct platform, fall through...
  )

;; HTML files needed for the operation of CG/JS
;; (this variable's value is generated by build_cgjs.sh)
(defvar *html-files* '("cgjs.html" "ssl-not-found.html" "browser-not-supported.html" ))

(defvar *extra-files*)
(setq *extra-files* nil)

(dolist (file *html-files*)
  (setq file (merge-pathnames file *load-pathname*))
  (or (probe-file file)
      (error "~a does not exist." file))
  (push file *extra-files*))
;; All *html-files* exist...

;; These two are directories but the pathname is specified as
;; a file, so merging works below.
(defvar *cg2js-directory* (merge-pathnames "code.cg2js" *load-pathname*))
(defvar *idejs-directory* (merge-pathnames "code.idejs" *load-pathname*))

(dolist (d (list *cg2js-directory* *idejs-directory*))
  (or (probe-file d)
      (error "~a does not exist." d)))

;; MD5 checksums of the incoming files
;; (this variable's value is generated by build_cgjs.sh)
(defvar *checksums* 
'(
(#x8fd9ab30527e2fbcc6627da36b552bc8 . "./code.cg2js/cg-acache.fasl")
(#x7751e686f11149860f71c789bdcf0677 . "./code.cg2js/cg-base.fasl")
(#x68b524c06a05cfbe1acbae2a2ae62c18 . "./code.cg2js/cg-bitmap-pane-clipboard.fasl")
(#x890b8c3ac022fb3ec1290437059477d3 . "./code.cg2js/cg-bitmap-pane.fasl")
(#xdba93639c05f76a533d7bc17f1b5d749 . "./code.cg2js/cg-bitmap-stream.fasl")
(#x9e1b0443e841ca370b28b7607b449954 . "./code.cg2js/cg-button.fasl")
(#x1459efa79a21515f6a8859e61a9b4aa1 . "./code.cg2js/cg-calendar.fasl")
(#xaa43259ad507e22d51bc26cbb07b2e22 . "./code.cg2js/cg-caret.fasl")
(#x4c1d52dae8233d1658b2703618fbb96c . "./code.cg2js/cg-chart-or-plot.fasl")
(#xa446b51dbadb7bcd140ddff5160d0210 . "./code.cg2js/cg-chart-widget.fasl")
(#x93a505663e6703f5c9c56d7ae393d5b8 . "./code.cg2js/cg-check-box.fasl")
(#xbf0928e95cd1f92d7dd00e957e6a03b9 . "./code.cg2js/cg-choice-list.fasl")
(#x760c1ca85a19320815b5fafbfab023cb . "./code.cg2js/cg-choose-printer.fasl")
(#xb85afe1862691211dc2cf851c24631b8 . "./code.cg2js/cg-class-grid.fasl")
(#x2d001bea5903440fe4fd3a230b5538c9 . "./code.cg2js/cg-class-slot-grid.fasl")
(#x131cbaeea1bd17afa71b1c9cfcb8d8f9 . "./code.cg2js/cg-class-support.fasl")
(#x48cdb96d74fa2d38d67fad4e674efd6d . "./code.cg2js/cg-clipboard-pixmap.fasl")
(#x5cd2e1547431c319d8512b574d17689b . "./code.cg2js/cg-clipboard-stack.fasl")
(#x5ea023dcaef278caf0e0ce131d191804 . "./code.cg2js/cg-clipboard.fasl")
(#x4c247cd70eadc466f7baa486dc2b2aa3 . "./code.cg2js/cg-color-dialog.fasl")
(#xbb4bc9f08fa392b136eb4152cf54c660 . "./code.cg2js/cg-combo-box.fasl")
(#x7ea3bddf789ef02f72a8083369a32f49 . "./code.cg2js/cg-common-control.fasl")
(#xe6d711ffc934e8344ee303dc6ce441fe . "./code.cg2js/cg-comtab.fasl")
(#x2f0cebf7a2d8fd54179d2145cfffa672 . "./code.cg2js/cg-cursor-pixmap.fasl")
(#xb6b2543031fc7d13f48a4802fa98dbd8 . "./code.cg2js/cg-curve.fasl")
(#x3169c1b7de11a0570fccfe54e9968ea6 . "./code.cg2js/cg-dde-utils.fasl")
(#x83eac73850a8d11554efaa52ae4719b1 . "./code.cg2js/cg-dev.fasl")
(#x519d88cda078e4b3b395b03e411c5c23 . "./code.cg2js/cg-dialog-item.fasl")
(#x20549c16cdc34874b5e622340f2a5d70 . "./code.cg2js/cg-directory-dialog-os.fasl")
(#xaa15d402830e8b27f6f6c8354c0bd462 . "./code.cg2js/cg-directory-dialog.fasl")
(#x38033f65c47208b940f5362e454c5696 . "./code.cg2js/cg-drag-and-drop-image.fasl")
(#xdb365464e9fe81b92926f1f413f57400 . "./code.cg2js/cg-drag-and-drop.fasl")
(#x6bb60c2923257df5254daa7a826f5154 . "./code.cg2js/cg-drawable-clipboard.fasl")
(#xbf279b2f44e2e420cf2792d0116f6cc0 . "./code.cg2js/cg-drawable.fasl")
(#x6e307757fb0b379c7e99582b9d94f1b9 . "./code.cg2js/cg-dropping-outline.fasl")
(#xb315fd833b1991bdb1be6826e7023e67 . "./code.cg2js/cg-edit-in-place.fasl")
(#xffef42b418b92a256b0dde123d9d3813 . "./code.cg2js/cg-editable-text.fasl")
(#x887020e78535e1e385a8f9b70a8ebc25 . "./code.cg2js/cg-file-dialog.fasl")
(#xc95c853b72fcb27d7f6bd2b396d0d8d5 . "./code.cg2js/cg-fill-texture.fasl")
(#xa6c7739e149e96c5dc6660b42bd58d1a . "./code.cg2js/cg-find-string-dialog.fasl")
(#xe15d539d788990395f4fee8793783f09 . "./code.cg2js/cg-font-dialog.fasl")
(#x59c1675a9bcca2d2fede26c3b067eac9 . "./code.cg2js/cg-gesture-emulation.fasl")
(#xcb484af7386cce283590e19c12d02e05 . "./code.cg2js/cg-get-pixmap.fasl")
(#x87cab9fe563b86edd255b33ffac46e31 . "./code.cg2js/cg-get-position.fasl")
(#xfb07172b27de6c316da9369679ff384e . "./code.cg2js/cg-graphics-context.fasl")
(#x7e708343361d1c0ded614334028435cc . "./code.cg2js/cg-grid-widget-drag-and-drop.fasl")
(#x6e7b0cc8b0cc9fc52a885b039ea55e0f . "./code.cg2js/cg-grid-widget.fasl")
(#x838c0c75344333f32aeb65bcbfe73d69 . "./code.cg2js/cg-group-box.fasl")
(#x75b73cf2553887c3c0232743fccfcd64 . "./code.cg2js/cg-header-control.fasl")
(#x71803e32c8401e4ca3ef8c19847bb5c7 . "./code.cg2js/cg-hotspot.fasl")
(#x48f1b47208a54d82c2c68324205d3c77 . "./code.cg2js/cg-html-dialog.fasl")
(#x6205e26c0b51545734a43e7600a6d97a . "./code.cg2js/cg-html-widget.fasl")
(#x9b11a481ac0cddf925beaf341af11315 . "./code.cg2js/cg-icon-pixmap.fasl")
(#xd760236fa24766e2779d8379ad608e5a . "./code.cg2js/cg-icon.fasl")
(#x3a91ab285a91fe7752b141735e8aa0b2 . "./code.cg2js/cg-ie.fasl")
(#x09b8019bca1b3b190ea58804c6ea6959 . "./code.cg2js/cg-intersect-posbox.fasl")
(#xc2c3dd79e3aed583a0bc74ad7c758aee . "./code.cg2js/cg-intersect.fasl")
(#xded7851b2e21ee675e439db00bb903fb . "./code.cg2js/cg-item-list.fasl")
(#xaeb2fb4f5f35794128a4df0c33844e80 . "./code.cg2js/cg-keyboard-shortcuts.fasl")
(#x0e559012b9707a7b65fff3dc50cefb40 . "./code.cg2js/cg-lamp.fasl")
(#xad9225e13a495b89de99ae43314d88f6 . "./code.cg2js/cg-layout.fasl")
(#x755f1e43851fcfe96d285ae14065c9e1 . "./code.cg2js/cg-lettered-menu.fasl")
(#x01c5b1c7f0712e1dfab4de56b757f6b9 . "./code.cg2js/cg-lisp-edit-pane.fasl")
(#x6014421018a764d77c6447c95bf8cba0 . "./code.cg2js/cg-lisp-text.fasl")
(#xc68bb606a4a8d1121eb690c2eeaba1a8 . "./code.cg2js/cg-lisp-widget.fasl")
(#xdd909fe1b3aefcbdbcd27eb0277fd023 . "./code.cg2js/cg-list-view.fasl")
(#xf87afb46f7afa9e253c4afb47f121134 . "./code.cg2js/cg-mci.fasl")
(#xb80e6bd4a1a02aa776ad0fff4eea0541 . "./code.cg2js/cg-menu-tooltip.fasl")
(#xd8e2f701648cb0479802b9199a3d57d3 . "./code.cg2js/cg-menu.fasl")
(#xaa9718ba8fc22d1abd76b22f025dfac4 . "./code.cg2js/cg-message-dialog.fasl")
(#x7b6316859af7ebfc64679d90c6405c0c . "./code.cg2js/cg-multi-line-editable-text.fasl")
(#xc2a7cae779011fd7c668a862cdc7a171 . "./code.cg2js/cg-multi-line-lisp-text.fasl")
(#xf63ef423343e8889c8d87b12c7dc8d7c . "./code.cg2js/cg-multi-picture-button-drag-and-drop.fasl")
(#x978bd7039da9e478b975af0eb0f14480 . "./code.cg2js/cg-multi-picture-button-tooltip.fasl")
(#x410f5b3bed896c91b7e6185d1af207be . "./code.cg2js/cg-multi-picture-button.fasl")
(#x345e4f10c2463a9cafb5ba86acdcfe76 . "./code.cg2js/cg-nodes.fasl")
(#xcfcdf97d7cedcfe0e93c9cf6b2bc7818 . "./code.cg2js/cg-object-editor-layout.fasl")
(#x6b4c1b0de6760bdcfc7292d1b348c742 . "./code.cg2js/cg-object-editor.fasl")
(#x7edb09bcb12ea901e867db4e21f3faa2 . "./code.cg2js/cg-ocx.fasl")
(#x699c210a761ebc2359871ea52c06600b . "./code.cg2js/cg-os-widget.fasl")
(#x6895d43235032f60def4003dfa6e59d1 . "./code.cg2js/cg-os-window.fasl")
(#x069f05b605abad5d144dc1f246b7ecfc . "./code.cg2js/cg-outline-drag-and-drop.fasl")
(#x7144f1987b69587e6af0085e3c9bfb35 . "./code.cg2js/cg-outline-edit-in-place.fasl")
(#x7ac2ae28edab11cc317cbf5946a1c888 . "./code.cg2js/cg-outline.fasl")
(#x8d2c739a96cc0b155cb2e519ce087f19 . "./code.cg2js/cg-palette.fasl")
(#xd44fbffa9c7628fd516af9ba06f5cad3 . "./code.cg2js/cg-paren-matching.fasl")
(#x7a9ff294abcf2842139e95c0292027ff . "./code.cg2js/cg-picture-widget-palette.fasl")
(#x88a1ce7d2b51f81198dc1b287311b173 . "./code.cg2js/cg-picture-widget.fasl")
(#x1f96d26936aa0e522321a2abf1250ce0 . "./code.cg2js/cg-pixmap-file-io.fasl")
(#xdea0bdf1d04b463455e0b33fd04298ba . "./code.cg2js/cg-pixmap-printing.fasl")
(#xf5d4ff40272e2165fc59cf6a61081d9e . "./code.cg2js/cg-pixmap-rotate.fasl")
(#xcd0e753db2a3f03b8e05bf93ad172d6e . "./code.cg2js/cg-pixmap-widget.fasl")
(#x0f80417e061e7d9961efc184e1cf84f7 . "./code.cg2js/cg-pixmap.fasl")
(#x7150dffa375680bb6e468f87675e4473 . "./code.cg2js/cg-plot-widget.fasl")
(#x8e9c057d67335958b35617520b62ce45 . "./code.cg2js/cg-printing.fasl")
(#x293a4dbec2500a2e926919c963a88efa . "./code.cg2js/cg-progress-indicator.fasl")
(#xa4b101f6e5591fc52e9cbb9f22e4b00f . "./code.cg2js/cg-project-window.fasl")
(#x200089fbc59d9582cade67b7b821e27a . "./code.cg2js/cg-property.fasl")
(#x2f7fbbf07fd2ef7fadc01d6e7616c279 . "./code.cg2js/cg-radio-button.fasl")
(#x27af31523593f0bc2195782dfe8b6791 . "./code.cg2js/cg-rich-edit-pane-clipboard.fasl")
(#x7f46ff67d888e4d4ca9ff57b283a4f93 . "./code.cg2js/cg-rich-edit-pane-printing.fasl")
(#x90fdf3c5c7410c3cfb7e2f9fae04b798 . "./code.cg2js/cg-rich-edit-pane.fasl")
(#x95b11dc349f925c368b156155eb10e78 . "./code.cg2js/cg-rich-edit.fasl")
(#xc39800831e623c12f182d015c37e1381 . "./code.cg2js/cg-sample-file-menu.fasl")
(#x372f275c9c91eb22f2b8a473b72e0dc1 . "./code.cg2js/cg-scaling-stream.fasl")
(#xc695f9b5b2a96a0cafbe87ab5d985730 . "./code.cg2js/cg-scroll-bar-mixin.fasl")
(#x04b6d5f577e96b34a5c91536c9b62ba8 . "./code.cg2js/cg-scroll-bar.fasl")
(#xf189c0b7d2a4239e4aebc0edf17c70e6 . "./code.cg2js/cg-scrolling-static-text.fasl")
(#xbb32a874c82935d22854e9a3619846c9 . "./code.cg2js/cg-selected-object.fasl")
(#xee9751b52404d113810086267b00cb47 . "./code.cg2js/cg-shortcut-menu.fasl")
(#xfaef3f5522c618727eb1018a427e1159 . "./code.cg2js/cg-split-bar.fasl")
(#x6304952c466921ea153088ecec74c708 . "./code.cg2js/cg-static-text.fasl")
(#xdb0ded92750e75faf10ceff68c6825a8 . "./code.cg2js/cg-status-bar.fasl")
(#xb12de37e8321572155ffc91f15640be6 . "./code.cg2js/cg-string-dialog.fasl")
(#x6e0fdd9270e8eca0886edf597c611640 . "./code.cg2js/cg-tab-control.fasl")
(#x5c291eefc99a94553e08e30183a3d86f . "./code.cg2js/cg-template-string.fasl")
(#x675b6466cf7760bb8c8d45c74c6994f6 . "./code.cg2js/cg-text-edit-pane-file-io.fasl")
(#x65546e22b4a307b09dac033d68ce42f4 . "./code.cg2js/cg-text-edit-pane-mark.fasl")
(#xda6bc061147a9da08f60be313176b15e . "./code.cg2js/cg-text-edit-pane.fasl")
(#x435bbd213e17c576177a2db68f04bbf0 . "./code.cg2js/cg-text-or-combo.fasl")
(#x02d2b6587453bf7df4c6565433f95026 . "./code.cg2js/cg-text-widget.fasl")
(#xa3fc0df56b53c59a63f6cea9f6c202d7 . "./code.cg2js/cg-timer.fasl")
(#x880ba1ea1d88d7ec7ce7ba0a133ec0aa . "./code.cg2js/cg-toggling-widget.fasl")
(#x72016b8d6dd9b7ae6e6fd5655dd33663 . "./code.cg2js/cg-toolbar.fasl")
(#x544bed5ce8f2d47be86a7a3556a5bc38 . "./code.cg2js/cg-tooltip.fasl")
(#xc31342213c27c11b96138ba8203d575d . "./code.cg2js/cg-touch.fasl")
(#x7b320d6d2a76a4500d3e1f47f2dafc85 . "./code.cg2js/cg-trackbar.fasl")
(#x2eec942078c1fcc2a8c30be7a7c880c8 . "./code.cg2js/cg-tray.fasl")
(#x7ed3dadb81aeb1531cf177f5ddad22d8 . "./code.cg2js/cg-up-down-control.fasl")
(#x58aac8979670c9b816539ac7e2be618a . "./code.cg2js/cg-utility-dialog.fasl")
(#x3e328ee3211d8746d4bae3b8bc505134 . "./code.cg2js/cg-web-browser-dde.fasl")
(#x8b6f8f243c78df50b85822ee8b17359e . "./code.cg2js/cg-web-browser.fasl")
(#xf687ab9650f8f67b6ece666fedefad7b . "./code.cg2js/cg-wrap-string.fasl")
(#x16b97dcb3af7dae4337df21354bf1b9b . "./code.cg2js/cg-yes-no-list.fasl")
(#x468c6f1eb64009918ab6d8df004e0db3 . "./code.cg2js/cg-yes-no-string.fasl")
(#xf2ad261c52e24c34eed82f30ddf3b0bb . "./code.cg2js/cg.fasl")
(#x73e029307d319e95c5e4f88d0bcf2075 . "./code.idejs/ide-acache.fasl")
(#x0909d01fbe7110ef1a476e602807e7eb . "./code.idejs/ide-apropos.fasl")
(#x6da9f4e660d491c2205ecd31b597f80e . "./code.idejs/ide-base.fasl")
(#xb480fb981644b80e472e797fabb800b0 . "./code.idejs/ide-class-browser.fasl")
(#xa1d8d4002927c086678c6e1856587ba6 . "./code.idejs/ide-class-grapher.fasl")
(#x11c862cbbe5ad8c06f27798f67cd9f36 . "./code.idejs/ide-clipboard.fasl")
(#x224be882beef861bc0d91c78453ac78a . "./code.idejs/ide-dev.fasl")
(#x93792722b98d17f60776b9b4bffb0aa7 . "./code.idejs/ide-editor.fasl")
(#xfe0cadc036f33cbdff99f239b91f5d5e . "./code.idejs/ide-find-definitions.fasl")
(#x28fad2a548fe4e2c3e087d88d37ec8f8 . "./code.idejs/ide-find-in-files.fasl")
(#xfc09079ad0529ec5c5bf1eb5671b231a . "./code.idejs/ide-form.fasl")
(#x764104cd74027be09f7f03793a65e233 . "./code.idejs/ide-grapher.fasl")
(#x54101ba833d53f75af16b1531edfdfc5 . "./code.idejs/ide-inspector.fasl")
(#xfb56f316c13261a22edd0b8f07298855 . "./code.idejs/ide-intro.fasl")
(#x6171c37e2974c1dbcdb2b4e6c3b86623 . "./code.idejs/ide-keyshort.fasl")
(#x70bf6bf5088419d98521d87169f9cb6e . "./code.idejs/ide-navigator.fasl")
(#x682dbac738e048477bc0dbdc0baf0db0 . "./code.idejs/ide-package-list.fasl")
(#x27769bfbf599bf9a00bf48a6d9e13932 . "./code.idejs/ide-patch-dialog.fasl")
(#x33c47cc55eddc64e8aef6a80f5afa426 . "./code.idejs/ide-process-browser.fasl")
(#x6934f5171cf91bd63790cd9117d17109 . "./code.idejs/ide-project.fasl")
(#x30496cb5a988780ff3ecb226c49f0472 . "./code.idejs/ide-trace.fasl")
(#x15b0c7396565b94e28888a0d92d68663 . "./code.idejs/ide-window-list.fasl")
(#x461766b8af286e86bd082dd587fa0ce5 . "./code.idejs/ide.fasl")
(#x82407180256cdbcae17acb711a3ed09f . "./cgjs.html")
(#xa0fd27890288fb85b9ee621ed47b2251 . "./ssl-not-found.html")
(#x4e1b21622d7d9cc4e6fc09c7875f8430 . "./browser-not-supported.html"))
 )

(let ((files-ok
       (dolist (checksum *checksums* t)
	 (when (/= (car checksum)
		   (md5-file
		    (merge-pathnames (cdr checksum) *load-pathname*)))
	   (return nil)))))
  (cond
   ((not files-ok)
    (error "The extracted files failed the checksum test.  ~
Please download and/or extract again."))
   (t
    (format t "Checksums on files are OK...~%"))))

;; We have validated all the files are there...

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; check for needed patches

(eval-when (compile eval load) (require :build))

(defvar *required-patches* '(#+macosx ("build" . 3)))

(dolist (required-patch *required-patches*)
  (let ((existing-patch (assoc (car required-patch)
			    (cdr (assoc :system sys::*patches*))
			    :test #'string=)))
    (when (null existing-patch)
      (error "Could not find patch info for the ~s module."
	     (car required-patch)))
    (when (not (>= (second existing-patch) (cdr required-patch)))
      ;; The build patch number is NOT >= to the one we require
      (error "~
Expected version ~d of ~s but have version ~d.  Please update ~
your lisp with update.~a and then do the CG/JS installation again."
	     (cdr required-patch) (car required-patch)
	     (second existing-patch)
	     #+mswindows "exe"
	     #-mswindows "sh"))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; copy files

(defun copy (source dest)
  (cond
   ((file-directory-p source)
    (format t "Copy directory ~a~%" (enough-namestring source))
    (copy-directory source dest :quiet t))
   (t
    (format t "Copy file ~a~%" (enough-namestring source))
    (sys:copy-file source dest :verbose nil))))

(defvar *sys* (translate-logical-pathname (pathname "sys:")))

(cond
 ((excl::pathname-equalp (truename *sys*)
			 (path-pathname *load-truename*))
  (format t "Files are already in place, no copying requiring.~%"))
 (t ;; copy files, replacing them with new ones if they already exist
  (let (dest)
    (dolist (from *extra-files*)
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      (when (probe-file dest) (delete-file dest))
      (copy from dest))
    
    (dolist (from (list *cg2js-directory* *idejs-directory*))
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      ;; now make it a directory pathname
      (setq dest (pathname-as-directory dest))

      (when (probe-file dest) (delete-directory-and-files dest))
      (copy (pathname-as-directory from) dest)))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; update develenv.cl to load CG/JS before the original version of CG

(defvar *setup-cgjs-loading* "sys:setup-cgjs-loading.cl")

;; Because users might use an ANSI lisp to install, have to make sure
;; the resulting file can be loaded into an ANSI and Modern lisp.
(with-open-file (s *setup-cgjs-loading* :direction :output
		 :if-exists :supersede)
  (format s "~
;; This file was created by install-cgjs.cl.
;; Please do not edit it.

")
  (dolist (dir (list "code.cg2js/" "code.idejs/"))
    (format s "
(let ((path (make-pathname
	     :type \"fasl\"
	     :directory (pathname-directory
			 (merge-pathnames
			  ~s
			  (translate-logical-pathname \"sys:\"))))))
  (format t \";; *require-search-list*: adding: ~~s~%\" path)
  (push path sys:*require-search-list*))

"
	    dir)))

(defvar *develenv.cl* (pathname "sys:develenv.cl"))

(when (not (probe-file *develenv.cl*))
  (setf (file-contents *develenv.cl*)
    ";; This file is loaded before images are rebuilt with update.sh/update.exe

"))

(let ((contents (file-contents *develenv.cl*)))
  (cond
   ((match-re
     (format nil "^#\\|\\|# \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl already correctly loads CG/JS.~%"))
   ((match-re
     (format nil "^#\\+ignore \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl: CG/JS disabled; will enable.~%")
    (setf (file-contents *develenv.cl*)
      (replace-re  contents
		  (format nil "^#\\+ignore *\\(load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  (format nil "#||# (load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  :multiple-lines t
		  :count 1)))
   (t
    (format t "develenv.cl: adding CG/JS initialization.~%")
    (setf (file-contents *develenv.cl*)
      (concatenate 'simple-string
	contents
	(format nil "~

;; To disable the use of CG/JS and go back to the originally released
;; (and possibly patched) version of CG/IDE, change the #||# to
;; #+ignore on the following line and the rebuild images by running
;; update.sh or update.exe, depending on your platform:
#||# (load \"sys:~a\")~%~%"
		(file-namestring *setup-cgjs-loading*)))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; success!

(let ((msg (format nil "~
CG/JS installation successful!

NOTE: to use CG/JS you must rebuild images by running 'update.~a'
      in the Allegro CL directory (~a)."
		   #+mswindows "exe"
		   #-mswindows "sh"
		   *sys*)))
  #-mswindows (format t "~&~%~a~%" msg)
  #+mswindows (excl::internal-message-box msg "CG/JS Installation" :type :ok))
